package cn.ibizlab.core.message;

import cn.ibizlab.core.extensions.domain.EngineMQMsg;
import cn.ibizlab.core.extensions.service.DABuildExService;
import cn.ibizlab.core.extensions.service.RuleEngineExService;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import lombok.extern.slf4j.Slf4j;
import org.apache.rocketmq.common.message.MessageExt;
import org.apache.rocketmq.spring.annotation.ConsumeMode;
import org.apache.rocketmq.spring.annotation.RocketMQMessageListener;
import org.apache.rocketmq.spring.core.RocketMQListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnExpression;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Component;

@Slf4j
@Component
@ConditionalOnExpression("'${rocketmq.consumer.namesrvAddr:}'!='' && ${rocketmq.consumer.enabled:false}")
@RocketMQMessageListener(nameServer = "${rocketmq.consumer.namesrvAddr:}",
        topic = "${rocketmq.producer.ruleEngineTopic:DSTMSG}",
        selectorExpression="*",
        consumerGroup = "${rocketmq.consumer.ruleEngineGroupName:dstCG}",
        consumeThreadMax = 1,
        consumeMode = ConsumeMode.ORDERLY
)
public class RuleEngineMQMsgListener implements RocketMQListener<MessageExt> {


    @Autowired
    @Lazy
    private RuleEngineExService ruleEngineExService;

    @Autowired
    @Lazy
    private DABuildExService daBuildExService;


    @Override
    public void onMessage(MessageExt messageExt) {

        try {
            log.info("MQ消息topic={}, tags={}, 消息内容={}", messageExt.getTopic(), messageExt.getTags(), messageExt.getMsgId());
            String body = new String(messageExt.getBody(), "utf-8");
            String tags = messageExt.getTags();

            EngineMQMsg engineMQMsg = JSON.parseObject(body, new TypeReference<EngineMQMsg>() {});
            System.out.println(engineMQMsg.toString());
            if("Engine".equalsIgnoreCase(tags)) {
                ruleEngineExService.processData(engineMQMsg);
            }else if("Build".equalsIgnoreCase(tags)){
                daBuildExService.processData(engineMQMsg);
            }
        } catch (Exception e) {
            log.error("获取MQ消息内容异常{}",e);
        }
    }

}